import subprocess
import pkg_resources
def check_envs():
    # for pwmat
    pass
    # check pwdata
    # try:
    #     package_version = pkg_resources.get_distribution('pwdata').version
    #     if pkg_resources.parse_version(min_version) <= pkg_resources.parse_version(package_version) <= pkg_resources.parse_version(max_version):
    #         print(f"{package_name} version {package_version} is within the required range [{min_version}, {max_version}].")
    #         return True
    #     else:
    #         print(f"{package_name} version {package_version} is NOT within the required range [{min_version}, {max_version}].")
    #         return False

    # check PWMLFF???
